/****************************************************************************
 * This file is part of the project AqHome.
 * AqHome (c) by 2025 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "./getdatapoints.h"
#include "../client.h"
#include "../utils.h"

#include "aqhome/msg/ipc/m_ipc.h"
#include "aqhome/msg/ipc/m_ipc_result.h"
#include "aqhome/msg/ipc/data/m_ipcd.h"
#include "aqhome/msg/ipc/data/m_ipcd_getdata.h"
#include "aqhome/msg/ipc/data/m_ipcd_multidata.h"

#include <gwenhywfar/args.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>

#include <ctype.h>



/* ------------------------------------------------------------------------------------------------
 * defs
 * ------------------------------------------------------------------------------------------------
 */

#define I18S(msg) msg
#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)

#define A_ARG GWEN_ARGS_FLAGS_HAS_ARGUMENT
#define A_END (GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST)
#define A_CHAR GWEN_ArgsType_Char
#define A_INT GWEN_ArgsType_Int



/* ------------------------------------------------------------------------------------------------
 * forward declarations
 * ------------------------------------------------------------------------------------------------
 */

static AQH_MESSAGE *_createRequestMessage(AQH_OBJECT *o, uint32_t msgId);
static int _handleResponseMessage(AQH_OBJECT *o, const AQH_MESSAGE *msg, const GWEN_TAG16_LIST *tagList, int first);
static void _handleDataResponse(const GWEN_TAG16_LIST *tagList, int printMean, int printDiff);



/* ------------------------------------------------------------------------------------------------
 * code
 * ------------------------------------------------------------------------------------------------
 */

int AQH_Tool_GetDataPoints(GWEN_DB_NODE *dbGlobalArgs, int argc, char **argv)
{
  AQH_EVENT_LOOP *eventLoop;
  AQH_OBJECT *o;
  int rv;
  const GWEN_ARGS args[]= {
    /* flags type    name                   min max s     long           short_descr, long_descr */
    { A_ARG, A_CHAR, "brokerAddress",       0,  1,  "t",  "tcpaddress",  I18S("TCP address to connect to [127.0.0.1]"), NULL},
    { A_ARG, A_INT,  "brokerPort",          0,  1,  "P",  "tcpport",     I18S("Specify the TCP port to listen on"), NULL},
    { A_ARG, A_INT,  "timeout",             0,  1,  "T",  NULL,          I18S("Specify timeout in seconds for response"), NULL},
    { A_ARG, A_CHAR, "brokerClientId",      0,  1,  "c",  "clientid",    I18S("Specify CLIENTID"), NULL},
    { A_ARG, A_CHAR, "userId",              0,  1,  "u",  "userid",      I18S("Specify user id"), NULL},
    { A_ARG, A_CHAR, "password",            0,  1,  "p",  "password",    I18S("Specify service password"), NULL},
    { A_ARG, A_CHAR, "valueName",           1,  1,  "N",  "valuename",   I18S("Value name (e.g. server/temp/system)"), NULL},
    { A_ARG, A_CHAR, "tsBegin",             0,  1,  "tb", "tsbegin",     I18S("Timestamp range begin"), NULL},
    { A_ARG, A_CHAR, "tsEnd",               0,  1,  "te", "tsend",       I18S("Timestamp range end"), NULL},
    { A_ARG, A_INT,  "numOfLastDatapoints", 0,  1,  "n",  NULL,          I18S("Get last n datapoints"), NULL},
    { 0,     A_INT,  "printMean",           0,  1,  "M",  "mean",        I18S("Print mean value of data received"), NULL},
    { 0,     A_INT,  "printDiff",           0,  1,  "D",  "diff",        I18S("Print diff last-first value"), NULL},
    { A_END, A_INT,  "help",                0,  0,  "h",  "help",        I18S("Show this help screen"), NULL}
  };

  eventLoop=AQH_EventLoop_new();
  o=AQH_ToolClient_new(eventLoop, AQH_IPC_PROTOCOL_DATA_ID, AQH_IPC_PROTOCOL_DATA_VERSION, dbGlobalArgs, args);
  AQH_ToolClient_AddFlags(o, AQH_TOOL_CLIENT_CONNECTFLAGS_WITHCONNECTMSG);
  AQH_ToolClient_SetCreateRequestMessageFn(o, _createRequestMessage);
  AQH_ToolClient_SetHandleResponseMessageFn(o, _handleResponseMessage);
  rv=AQH_ToolClient_ReadLocalArgs(o, argc, argv);
  if (rv!=0)
    return rv;
  rv=AQH_ToolClient_Run(o);
  AQH_Object_free(o);
  AQH_EventLoop_free(eventLoop);
  return rv;
}



AQH_MESSAGE *_createRequestMessage(GWEN_UNUSED AQH_OBJECT *o, uint32_t msgId)
{
  GWEN_DB_NODE *dbArgs;
  const char *valueName;
  uint64_t tsBegin;
  uint64_t tsEnd;
  uint64_t num;

  dbArgs=AQH_ToolClient_GetDbLocalArgs(o);
  valueName=GWEN_DB_GetCharValue(dbArgs, "valueName", 0, NULL);
  num=GWEN_DB_GetIntValue(dbArgs, "numOfLastDatapoints", 0, 0);
  tsBegin=Utils_GetTimeStampFromString(GWEN_DB_GetCharValue(dbArgs, "tsBegin", 0, NULL));
  if (tsBegin==(uint64_t) (-1)) {
    DBG_ERROR(NULL, "Bad begin timestamp");
    return NULL;
  }
  tsEnd=Utils_GetTimeStampFromString(GWEN_DB_GetCharValue(dbArgs, "tsEnd", 0, NULL));
  if (tsEnd==(uint64_t) (-1)) {
    DBG_ERROR(NULL, "Bad end timestamp");
    return NULL;
  }

  // TODO: use "mode" correctly
  return AQH_IpcdMessageGetData_new(AQH_MSGTYPE_IPC_DATA_GETDATA_REQ,
                                    msgId, 0,
                                    AQH_MSGDATA_GETDATA_MODE_LAST,
                                    valueName,
                                    tsBegin, tsEnd, num);
}



int _handleResponseMessage(AQH_OBJECT *o, const AQH_MESSAGE *msg, const GWEN_TAG16_LIST *tagList, int first)
{
  GWEN_DB_NODE *dbArgs;
  int printMean;
  int printDiff;
  uint16_t code;

  dbArgs=AQH_ToolClient_GetDbLocalArgs(o);
  printMean=GWEN_DB_GetIntValue(dbArgs, "printMean", 0, 0);
  printDiff=GWEN_DB_GetIntValue(dbArgs, "printDiff", 0, 0);

  code=AQH_IpcMessage_GetCode(msg);
  if (code==AQH_MSGTYPE_IPC_DATA_GETDATA_RSP) {
    _handleDataResponse(tagList, printMean, printDiff);
    return 1;
  }
  else if (code==AQH_MSGTYPE_IPC_DATA_RESULT)
    return AQH_ToolClient_HandleResultMsg(msg, tagList, first);
  else {
    DBG_INFO(NULL, "Unexpected message \"%d\"", code);
    return 3;
  }
}



void _handleDataResponse(const GWEN_TAG16_LIST *tagList, int printMean, int printDiff)
{
  AQH_VALUE *value;

  value=AQH_IpcdMessageMultiData_ReadValue(tagList);
  if (value) {
    const char *valueUnits;
    uint64_t numberOfPoints;
    const uint64_t *dataPoints;

    valueUnits=AQH_Value_GetValueUnits(value);
    AQH_IpcdMessageMultiData_ReadDatapoints(tagList, &dataPoints, &numberOfPoints);
    if (numberOfPoints>0 && dataPoints) {
      if (printMean)
        Utils_PrintMeanData(dataPoints, numberOfPoints, valueUnits);
      else if (printDiff)
        Utils_PrintDiffData(dataPoints, numberOfPoints, valueUnits);
      else
        Utils_PrintDataPoints(dataPoints, numberOfPoints, valueUnits);
    }
    AQH_Value_free(value);
  }
}



